from setuptools import setup

setup(
    name='sig_process_arrays',
    version='1.0',
    packages=['sig_process_arrays'],
    url='',
    license='MIT license',
    author='Sammy Kolpinizki',
    author_email='sammykol@gmail.com',
    description='Plots magnitude of beam response for 1D receiver arrays - This is a "Portfolio" exercise from the Udacity Nano Degree in Datasciene '
)
